#version 150

#moj_import <fog.glsl>

uniform vec4 ColorModulator;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;
uniform vec2 ScreenSize;

uniform float GameTime;

out vec4 fragColor;

flat in int skySpeed;
in mat4 ProjInv;
in float isSky;
in float vertexDistance;

#moj_import <nlz_utils.glsl>
#moj_import <nlz_ab_speeds.glsl>


#define TEST 0

#define STARS_SPEED 600

#define MSTARS_SIZE .5
#define MSTARS_SPEED 120.


vec4 mapStars(float time, vec3 view, float bsize, float angleOffset, int skySpeed) {
	vec2 pos = vec2(view.x, view.y);
	bool rev = (view.z > 0);
	if (rev) {
		time = 2*PI - time;
	}


	vec4 color = vec4(vec3(0.), 1.);
	float angle = atan(pos.y, pos.x) + angleOffset;
	float angleNumber = 20.;
	float random = rand(floor(angle * angleNumber) * 100.)* speedParams[skySpeed].w;
	float dist = random;
	dist += time * speedParams[skySpeed].y;
	dist = fract(dist);

	float fragD = length(pos);
	float size = smoothstep(-.1, 2., bsize);
	float bounded = step(dist, fragD) * step(fragD, dist + size);

	bounded += step(fragD, fract(dist + size)) * step(fract(dist + size), dist);    // non continuous


	color.rgb = mix(vec3(0.), vec3(1.), bounded);
	color.r *= 1.;
	color.g *= .7 + (random/speedParams[skySpeed].w) * .3;    // simulate distance
	color.b *= 4.;

	// hard edges
	float angleCenter = abs(fract(angle * angleNumber) * 2. - 1.);
	color.a = step(angleCenter, smoothstep(-.2, 1., bsize));

	// smooth gradient
	color.a *= 1. - angleCenter;
	color.a *= smoothstep(0., 1., fragD);
	color.a *= cos(random * time) * .5 + .5;

	// filter
	if (color.r+color.g+color.b == 0.0)
		color.a = 0.0;

	return color;
}


void main() {
	// if (skySpeed == -1) {
	// 	fragColor = vec4(FogColor.xyz, 1.);
	// 	// fragColor = vec4(1., 0., 0., 1.);
	// 	return;
	// } else {
	// 	fragColor = debugColors[skySpeed];
	// 	// fragColor = vec4(1., 0., 0., 1.);
	// 	return;
	// }

	float t = GameTime * 2 * PI;

	const vec4 skyColor = vec4(0.14, 0, 0.31, 1);

	// get player space view vector
	vec4 screenPos = gl_FragCoord;
	screenPos.xy = (screenPos.xy / ScreenSize - vec2(0.5)) * 2.0;
	screenPos.zw = vec2(1.0);
	vec3 view = normalize((ProjInv * screenPos).xyz);
	
	if (isSky > 0.5) {
		fragColor = skyColor;

		// light speed stars
		vec4 col = mapStars(t, view, speedParams[skySpeed].x, 20., skySpeed);//* .5 + smoothstep(.9, 1., speedParams[skySpeed].x);
		fragColor = mix(fragColor, col, col.a);
	} 
	else { // stars 
		float posCoef = ((view.y + view.x + view.z) / 3.0) + .5;
		fragColor = vec4(mix(skyColor.xyz, vec3(1), (cos(t*speedParams[skySpeed].z*0.3 + posCoef*100)/2)+0.5), (sin(t*speedParams[skySpeed].z + posCoef*100)/2)+0.6);
	}
}
